--[[ NavHelper by MrInfernal  v2.3

GUI to improve usage of MQ2Nav and streamline travel
Allows the user to search by zone full name, short name, or ID and returns
a selectable table. Click a zone and you can Start, Pause, or Cancel travel.

Buttons to check for teleport locations near your selected zone and to cast the port spell.
]]

---@type Mq
local mq = require('mq')
---@type ImGui
local imgui = require 'ImGui'

-- Import the zone and port files
DruPorts = require('druports')
WizPorts = require('wizports')

AllPorts = {}
for _, addwiz in ipairs(WizPorts) do
    table.insert(AllPorts, addwiz)
end
for _, adddru in ipairs(DruPorts) do
    table.insert(AllPorts, adddru)
end

-- Set default values
local stayOpen = true
local portSpell = ""        -- the port spell to cast
local useCWTN = false       -- Disable if not using CWTN Druid

-- Add druids and wizards to a DanNet channel called porters
if mq.TLO.Me.Class() == 'Druid' then
    mq.cmd('/squelch /djoin porters')
elseif mq.TLO.Me.Class() == 'Wizard' then
    mq.cmd('/squelch /djoin porters')
else
    mq.cmd('/squelch /dge dru /djoin porters')
    mq.cmd('/squelch /dge wiz /djoin porters')
end

-- Create a table of the porter character name and class checks
local function setMyPorters(porters)
    local myPorters = {}

    if mq.TLO.Group() then
        for char in porters:gmatch("[^|]+") do
            local name = mq.TLO.Group.Member(char).Name()
            local class = mq.TLO.Group.Member(char).Class()
            local level = mq.TLO.Group.Member(char).Level()
            table.insert(myPorters, {name = name, class = class, level = level})
        end
    else
        for char2 in porters:gmatch("[^|]+") do
            local name = mq.TLO.Me.Name()
            local class = mq.TLO.Me.Class()
            local level = mq.TLO.Me.Level()
            table.insert(myPorters, {name = name, class = class, level = level})
        end
    end

    return myPorters
end

-- Use the myPorters table to check for port classes
-- This will set which port spells display (Dru/Wiz/Both)
local myPorters = setMyPorters(tostring(mq.TLO.DanNet.Peers))

local hasDruid = false
local hasWizard = false

for i, porter in ipairs(myPorters) do
    if porter.class == "Druid" then
        hasDruid = true
    elseif porter.class == "Wizard" then
        hasWizard = true
    end
end

local function checkCWTN()
    if mq.TLO.Plugin('mq2druid').IsLoaded() and hasDruid then
        useCWTN = true
    else
        useCWTN = false
    end
end

local function doEvac()
    local evacSpells = {'Exodus', 'Succor', 'Lesser Succor', 'Evacuate', 'Lesser Evacuate'}
    local portName = ''
    local myPorters = setMyPorters(tostring(mq.TLO.DanNet.Peers))

    for _, char in ipairs(myPorters) do
        for _, port in ipairs(evacSpells) do
            if char.class == 'Druid' and char.level >= 57 and port == 'Succor' then
                portName = port
                break
            elseif char.class == 'Druid' and char.level >= 18 and port == 'Lesser Succor' then
                portName = port
                break
            elseif char.class == 'Wizard' and char.level >= 57 and port == 'Evacuate' then
                portName = port
                break
            elseif char.class == 'Wizard' and char.level >= 18 and port == 'Lesser Evacuate' then
                portName = port
                break
            end
        end
    end

    if hasDruid and useCWTN then
        if mq.TLO.Me.Class.ShortName() == 'DRU' then
            mq.cmd('/squelch /dru pause on')
        else
            mq.cmd('/squelch /dgae porters /dru pause on')
        end
    end

    -- Have the porter cast the spell. Will report if need to memorize or don't have the spell
    mq.cmd('/noparse /squelch /dgae porters /if (${Cast.Ready[exodus]}) /casting "exodus" alt')
    mq.cmdf('/timed 1 /noparse /squelch /dgae porters /if (${Me.Book["%s"]}>0) /casting "%s"', portName, portName)
    mq.cmdf('/timed 2 /noparse /squelch /dex %s /if (!${Me.Book["%s"]}>0) /dgt all Spell not known!', myPorters[1].name, portName)

    -- Have to memorize
    mq.cmdf('/timed 5 /noparse /squelch /dgae porters /if (${Cast.Status.Equal[CM]}) /timed 5 /dgt all Memorizing "%s"! Ready in 1.5 seconds', portName)
    mq.cmdf([[/timed 50 /noparse /squelch /dgae porters /if (${Cast.Timing}>6000) /dgt all Casting -> %s <- in ${Math.Calc[${Cast.Timing}/1000]} seconds!]], portName)
    mq.cmdf([[/timed 90 /noparse /squelch /dgae porters /if (${Cast.Timing}>4000) /dgt all ${Math.Calc[${Cast.Timing}/1000]} seconds remaining!]], portName)

    -- Already memorized
    mq.cmdf('/timed 5 /noparse /squelch /dgae porters /if (${Cast.Timing}>8000) /dgt all Casting -> %s <- in ${Math.Calc[${Cast.Timing}/1000]} seconds!', portName)
    mq.cmdf('/timed 40 /noparse /squelch /dgae porters /if (${Cast.Timing}>5000) /dgt all ${Math.Calc[${Cast.Timing}/1000]} seconds remaining!', portName)

    -- Out of mana
    mq.cmdf('/timed 5 /noparse /squelch /dgae porters /if (${Cast.Status.Equal[CAST_OUTOFMANA]}) /dgt all Out of mana!')
    --mq.cmdf('/timed 8 /noparse /squelch /dgae porters /if (${Cast.Status.Equal[CAST_OUTOFMANA]}) /if (!${Me.Sitting}) /sit')
end

local function exitProg()
    stayOpen = false
end

-- Create a function to handle the teleporting
local function boundPort(input)
    local found = false
    local portName = ''

    for _, port in ipairs(AllPorts) do
        if string.lower(port.name) == string.lower(input) then
            found = true
            portName = port.name
            break
        end
    end

    if found then
        if useCWTN then
            if mq.TLO.Me.Class() == 'Druid' then
                mq.cmd('/squelch /dgae porters /dru pause on')
                mq.cmd('/dgt all Pausing MQ2Druid!')
            else
                if hasDruid then
                    mq.cmd('/squelch /dge porters /dru pause on')
                    mq.cmd('/dgt all Pausing MQ2Druid!')
                end
            end
        end
        if mq.TLO.Me.Class() == 'Druid' then
            mq.cmdf('/squelch /dgae porters /casting "%s"', portName)
            mq.cmdf('/dgt all Casting "%s"!', portName)
        else
            -- Have the porter cast the spell. Will report if need to memorize or don't have the spell
            mq.cmdf('/noparse /squelch /dgae porters /if (${Me.Book["%s"]}>0) /casting "%s"', portName, portName)
            mq.cmdf('/noparse /squelch /dex %s /if (!${Me.Book["%s"]}>0) /dgt all Spell not known!', myPorters[1].name, portName)

            -- Have to memorize
            mq.cmdf('/timed 5 /noparse /squelch /dgae porters /if (${Cast.Status.Equal[CM]}) /timed 5 /dgt all Memorizing "%s"! Ready in 10 seconds', portName)
            mq.cmdf([[/timed 140 /noparse /squelch /dgae porters /if (${Cast.Timing}>2000) /dgt all Casting -> %s <- in ${Math.Calc[${Cast.Timing}/1000]} seconds!]], portName)
            mq.cmdf([[/timed 230 /noparse /squelch /dgae porters /if (${Cast.Timing}>5000) /dgt all Casting -> %s <- in ${Math.Calc[${Cast.Timing}/1000]} seconds!]], portName)

            -- Already memorized
            mq.cmdf('/timed 5 /noparse /squelch /dgae porters /if (${Cast.Timing}>2000) /dgt all Casting -> %s <- in ${Math.Calc[${Cast.Timing}/1000]} seconds!', portName)
            mq.cmdf('/timed 80 /noparse /squelch /dgae porters /if (${Cast.Timing}>5000) /dgt all ${Math.Calc[${Cast.Timing}/1000]} seconds remaining!', portName)

            -- Out of mana
            mq.cmdf('/timed 5 /noparse /squelch /dgae porters /if (${Cast.Status.Equal[CAST_OUTOFMANA]}) /dgt all Out of mana!')
            mq.cmdf('/timed 8 /noparse /squelch /dgae porters /if (${Cast.Status.Equal[CAST_OUTOFMANA]}) /if (!${Me.Sitting}) /sit')
        end
    else
        print('Spell not found!')
    end
end

-- Commands to run after zoning
local function justZoned()
    if useCWTN then
        if mq.TLO.Me.Class() == 'Druid' then
            mq.cmd('/squelch /dgae porters /dru pause off')
            mq.cmd('/dgt all Unpausing MQ2Druid!')
        else
            for _, char in ipairs(myPorters) do
                if char.class == 'Druid' then
                    mq.cmdf('/squelch /dex %s /dru pause off', char.name)
                    mq.cmdf('/squelch /dex %s /dgt all Unpausing MQ2Druid!', char.name)
                --[[elseif char.class == 'Wizard' then
                    mq.cmdf('/squelch /dex %s /rgstart', char.name)
                    mq.cmdf('/squelch /dex %s /dgt all Restarting RGMercs macro!', char.name)]]
                end
            end
        end
    end
end
-------------- End of functions

-- Creates slash commands for porting. Allows creation of hotkeys
-- ex: /nh "circle of knowledge"
mq.bind("/nh", boundPort)
mq.bind('/nhevac', doEvac)
mq.bind('/nhexit', exitProg)

-- Set up event listeners
mq.event('zoned', 'You have entered#*#', justZoned)

-- See if MQ2Druid is loaded and a grouped druid. If so, then toggle the useCWTN box in settings
checkCWTN()

while stayOpen do
    mq.doevents()
    mq.delay(1000) -- equivalent to '1s'
end